@@ -1,13 +1,16 @@
 /*
 	Cabinet.cc
 	
-	Copyright 2002-4 Tim Goetze <tim@quitte.de>
+	Copyright 2002-5 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
-	16th order IIR filters modeled after various impulse responses from
-	Steve Harris' 'imp' plugin.
+	CabinetI - 16th order IIR filters modeled after various impulse responses 
+	from Steve Harris' 'imp' plugin. Limited to 44.1 kHz sample rate.
 
+	CabinetII - 32nd order IIR filters modeled after the same impulse responses
+	using a different algorithm. Versions for 44.1 / 48 / 88.2 / 96 kHz sample
+	rates, switched at runtime.
 */
 /*
 	This program is free software; you can redistribute it and/or
@@ -31,8 +34,8 @@
 #include "Cabinet.h"
 #include "Descriptor.h"
 
-Model
-Cabinet::models [] = 
+Model16
+CabinetI::models [] = 
 {
 	{
 		1, /* identity */
@@ -70,7 +73,7 @@ Cabinet::models [] =
 /* //////////////////////////////////////////////////////////////////////// */
 
 void
-Cabinet::init (double fs)
+CabinetI::init (double fs)
 {
 	h = 0;
 	model = 0;
@@ -78,28 +81,32 @@ Cabinet::init (double fs)
 }
 
 void
-Cabinet::switch_model (int m)
+CabinetI::switch_model (int m)
 {
+	if (m < 0) m = 0;
+	else if (m > 5) m = 5;
+
 	model = m;
 
 	n = models[m].n;
 	a = models[m].a;
 	b = models[m].b;
 
+	gain = models[m].gain * DSP::db2lin (*ports[2]);
+
 	memset (x, 0, sizeof (x));
 	memset (y, 0, sizeof (y));
 }
 
 void
-Cabinet::activate()
+CabinetI::activate()
 {
 	switch_model ((int) *ports[1]);
-	gain = models[model].gain * DSP::db2lin (*ports[2]);
 }
 
 template <sample_func_t F>
 void
-Cabinet::one_cycle (int frames)
+CabinetI::one_cycle (int frames)
 {
 	d_sample * s = ports[0];
 
@@ -140,7 +147,7 @@ Cabinet::one_cycle (int frames)
 /* //////////////////////////////////////////////////////////////////////// */
 
 PortInfo
-Cabinet::port_info [] =
+CabinetI::port_info [] =
 {
 	{
 		"in",
@@ -149,7 +156,7 @@ Cabinet::port_info [] =
 	}, {
 		"model",
 		INPUT | CONTROL,
-		{BOUNDED | INTEGER | DEFAULT_1, 0, sizeof (models) / sizeof (Model) - .5}
+		{BOUNDED | INTEGER | DEFAULT_1, 0, 5}
 	}, {
 		"gain (dB)",
 		INPUT | CONTROL,
@@ -161,16 +168,145 @@ Cabinet::port_info [] =
 	}
 };
 
-void
-Descriptor<Cabinet>::setup()
+/* //////////////////////////////////////////////////////////////////////// */
+
+template <> void
+Descriptor<CabinetI>::setup()
 {
 	UniqueID = 1766;
-	Label = "Cabinet";
+	Label = "CabinetI";
 	Properties = HARD_RT;
 
-	Name = "Amp/speaker emulation";
+	Name = "CAPS: CabinetI - Loudspeaker cabinet emulation";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2002-4";
+	Copyright = "GPL, 2002-5";
+
+	/* fill port info and vtable */
+	autogen();
+}
+
+/* CabinetII ////////////////////////////////////////////////////////////// */
+
+#include "Cabinet-Models32.h"
+
+void
+CabinetII::init (double fs)
+{
+	if (fs < 46000)
+		models = models44100;
+	else if (fs < 72000)
+		models = models48000;
+	else if (fs < 92000)
+		models = models88200;
+	else 
+		models = models96000;
+
+	h = 0;
+	model = 0;
+	normal = NOISE_FLOOR;
+}
+
+void
+CabinetII::switch_model (int m)
+{
+	if (m < 0) m = 0;
+	else if (m > 5) m = 5;
+
+	model = m;
+
+	n = models[m].n;
+	a = models[m].a;
+	b = models[m].b;
+
+	gain = models[m].gain * DSP::db2lin (*ports[2]);
+
+	memset (x, 0, sizeof (x));
+	memset (y, 0, sizeof (y));
+}
+
+void
+CabinetII::activate()
+{
+	switch_model ((int) *ports[1]);
+	gain = models[model].gain * DSP::db2lin (*ports[2]);
+}
+
+template <sample_func_t F>
+void
+CabinetII::one_cycle (int frames)
+{
+	d_sample * s = ports[0];
+
+	int m = (int) *ports[1];
+	if (m != model) switch_model (m);
+
+	d_sample g = models[model].gain * DSP::db2lin (*ports[2]);
+	double gf = pow (g / gain, 1 / (double) frames);
+
+	d_sample * d = ports[3];
+
+	for (int i = 0; i < frames; ++i)
+	{
+		register cabinet_float out = s[i] + normal;
+		
+		x[h] = out;
+		
+		out *= a[0];
+
+		for (int j = 1, z = h - 1; j < n; --z, ++j)
+		{
+			z &= 31;
+			out += a[j] * x[z];
+			out += b[j] * y[z];
+		}
+
+		y[h] = out;
+
+		h = (h + 1) & 31;
+		
+		F (d, i, gain * out, adding_gain);
+		gain *= gf;
+	}
+
+	normal = -normal;
+}
+
+/* //////////////////////////////////////////////////////////////////////// */
+
+PortInfo
+CabinetII::port_info [] =
+{
+	{
+		"in",
+		INPUT | AUDIO,
+		{BOUNDED, -1, 1}
+	}, {
+		"model",
+		INPUT | CONTROL,
+		{BOUNDED | INTEGER | DEFAULT_1, 0, 5}
+	}, {
+		"gain (dB)",
+		INPUT | CONTROL,
+		{BOUNDED | DEFAULT_0, -24, 24}
+	}, {
+		"out",
+		OUTPUT | AUDIO,
+		{0}
+	}
+};
+
+/* //////////////////////////////////////////////////////////////////////// */
+
+template <> void
+Descriptor<CabinetII>::setup()
+{
+	UniqueID = 2581;
+	Label = "CabinetII";
+	Properties = HARD_RT;
+
+	Name = "CAPS: CabinetII - Refined loudspeaker cabinet emulation";
+	Maker = "Tim Goetze <tim@quitte.de>";
+	Copyright = "GPL, 2002-5";
 
 	/* fill port info and vtable */
 	autogen();
