@@ -1,11 +1,11 @@
 /*
 	Click.cc
 	
-	Copyright 2002-4 Tim Goetze <tim@quitte.de>
+	Copyright 2002-5 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
-	plays a click in regular intervals.
+	Plugins playing a sound snippet in regular intervals.
 
 */
 /*
@@ -125,16 +125,16 @@ Click::init (double fs)
 	this->ClickStub::init (fs, click, LENGTH (click));
 }
 
-void
+template <> void
 Descriptor<Click>::setup()
 {
 	UniqueID = 1769;
 	Label = "Click";
 	Properties = HARD_RT;
 
-	Name = "Metronome";
+	Name = "CAPS: Click - Metronome";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2004";
+	Copyright = "GPL, 2004-5";
 
 	/* fill port info and vtable */
 	autogen();
@@ -170,16 +170,63 @@ CEO::init (double fs)
 	this->ClickStub::init (fs, money, LENGTH (money));
 }
 
-void
+template <> void
 Descriptor<CEO>::setup()
 {
 	UniqueID = 1770;
 	Label = "CEO";
 	Properties = HARD_RT;
 
-	Name = "Chief Executive Oscillator";
+	Name = "CAPS: CEO - Chief Executive Oscillator";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2004";
+	Copyright = "GPL, 2004-5";
+
+	/* fill port info and vtable */
+	autogen();
+}
+
+/* //////////////////////////////////////////////////////////////////////// */
+
+float dirac [] = { 1, };
+
+PortInfo
+Dirac::port_info [] =
+{
+	{
+		"ppm",
+		INPUT | CONTROL,
+		{BOUNDED | DEFAULT_MIN, 30, 60}
+	}, {
+		"volume",
+		INPUT | CONTROL,
+		{BOUNDED | DEFAULT_0, 0, 1}
+	}, {
+		"damping",
+		INPUT | CONTROL,
+		{BOUNDED | DEFAULT_0, 0, 1}
+	}, {
+		"out",
+		OUTPUT | AUDIO,
+		{0}
+	}
+};
+
+void
+Dirac::init (double fs)
+{ 
+	this->ClickStub::init (fs, dirac, LENGTH (dirac));
+}
+
+template <> void
+Descriptor<Dirac>::setup()
+{
+	UniqueID = 2585;
+	Label = "Dirac";
+	Properties = HARD_RT;
+
+	Name = "CAPS: Dirac - One-sample impulse generator";
+	Maker = "Tim Goetze <tim@quitte.de>";
+	Copyright = "GPL, 2004-5";
 
 	/* fill port info and vtable */
 	autogen();
