@@ -1,7 +1,7 @@
 /*
 	Pan.cc
 	
-	Copyright 2002-4 Tim Goetze <tim@quitte.de>
+	Copyright 2002-5 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
@@ -145,16 +145,16 @@ Pan::port_info [] =
 	}
 };
 
-void
+template <> void
 Descriptor<Pan>::setup()
 {
 	UniqueID = 1788;
 	Label = "Pan";
 	Properties = HARD_RT;
 
-	Name = "Pan and width";
+	Name = "CAPS: Pan - Pan and width";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2004";
+	Copyright = "GPL, 2004-5";
 
 	/* fill port info and vtable */
 	autogen();
