@@ -1,7 +1,7 @@
 /*
 	Sin.cc
 	
-	Copyright 2002-4 Tim Goetze <tim@quitte.de>
+	Copyright 2002-5 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
@@ -79,16 +79,16 @@ Sin::port_info [] =
 	}
 };
 
-void
+template <> void
 Descriptor<Sin>::setup()
 {
 	UniqueID = 1781;
 	Label = "Sin";
 	Properties = HARD_RT;
 
-	Name = "Sine generator";
+	Name = "CAPS: Sin - Sine wave generator";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2004";
+	Copyright = "GPL, 2004-5";
 
 	/* fill port info and vtable */
 	autogen();
