@@ -1,7 +1,7 @@
 /*
 	White.cc
 	
-	Copyright 2004 Tim Goetze <tim@quitte.de>
+	Copyright 2004-5 Tim Goetze <tim@quitte.de>
 	
 	http://quitte.de/dsp/
 
@@ -64,16 +64,16 @@ White::port_info [] =
 	}
 };
 
-void
+template <> void
 Descriptor<White>::setup()
 {
 	UniqueID = 1785;
 	Label = "White";
 	Properties = HARD_RT;
 
-	Name = "White noise generator";
+	Name = "CAPS: White - White noise generator";
 	Maker = "Tim Goetze <tim@quitte.de>";
-	Copyright = "GPL, 2004";
+	Copyright = "GPL, 2004-5";
 
 	/* fill port info and vtable */
 	autogen();
